@@ -13,124 +13,140 @@ use Carp;
 use Data::Dumper qw/Dumper/;
 use English qw/ -no_match_vars /;
 use base qw/VCS::Which::Plugin/;
+use Path::Class;
 use File::chdir;
 use Contextual::Return;
-use Path::Class;
 
-our $VERSION = version->new('0.2.0');
+our $VERSION = version->new('0.3.0');
 our $name    = 'Subversion';
 our $exe     = 'svn';
 our $meta    = '.svn';
 
 sub installed {
-	my ($self) = @_;
+    my ($self) = @_;
 
-	return $self->{installed} if exists $self->{installed};
+    return $self->{installed} if exists $self->{installed};
 
-	for my $path (split /[:;]/, $ENV{PATH}) {
-		next if !-x "$path/$exe";
+    for my $path (split /[:;]/, $ENV{PATH}) {
+        next if !-x "$path/$exe";
 
-		return $self->{installed} = 1;
-	}
+        return $self->{installed} = 1;
+    }
 
-	return $self->{installed} = 0;
+    return $self->{installed} = 0;
 }
 
 sub used {
-	my ( $self, $dir ) = @_;
+    my ( $self, $dir ) = @_;
 
-	if (-f $dir) {
-		$dir = file($dir)->parent;
-	}
+    if (-f $dir) {
+        $dir = file($dir)->parent;
+    }
 
-	croak "$dir is not a directory!" if !-d $dir;
+    croak "$dir is not a directory!" if !-d $dir;
 
-	return -d "$dir/$meta";
+    return -d "$dir/$meta";
 }
 
 sub uptodate {
-	my ($self, $dir) = @_;
+    my ( $self, $dir ) = @_;
 
-	$dir ||= $self->{base};
+    $dir ||= $self->{base};
 
-	croak "'$dir' is not a directory!" if !-e $dir;
+    croak "'$dir' is not a directory!" if !-e $dir;
 
-	my @lines = `$exe status $dir`;
-	pop @lines;
+    my @lines = `$exe status $dir`;
+    pop @lines;
 
-	return scalar @lines;
+    return !@lines;
 }
 
 sub pull {
-	my ( $self, $dir ) = @_;
+    my ( $self, $dir ) = @_;
 
-	$dir ||= $self->{base};
+    $dir ||= $self->{base};
 
-	croak "'$dir' is not a directory!" if !-e $dir;
+    croak "'$dir' is not a directory!" if !-e $dir;
 
-	local $CWD = $dir;
-	return !system "$exe update > /dev/null 2> /dev/null";
+    local $CWD = $dir;
+    return !system "$exe update > /dev/null 2> /dev/null";
 }
 
 sub cat {
-	my ($self, $file, $revision) = @_;
+    my ($self, $file, $revision) = @_;
 
-	if ( $revision && $revision =~ /^-\d+$/xms ) {
-		my @versions = reverse `$exe log -q $file` =~ /^ r(\d+) \s/gxms;
-		$revision = $versions[$revision];
-	}
-	elsif ( !defined $revision ) {
-		$revision = '';
-	}
+    if ( $revision && $revision =~ /^-\d+$/xms ) {
+        my @versions = reverse `$exe log -q $file` =~ /^ r(\d+) \s/gxms;
+        $revision = $versions[$revision];
+    }
+    elsif ( !defined $revision ) {
+        $revision = '';
+    }
 
-	$revision &&= "-r$revision";
+    $revision &&= "-r$revision";
 
-	return `$exe cat $revision $file`;
+    return `$exe cat $revision $file`;
 }
 
 sub log {
-	my ($self, @args) = @_;
+    my ($self, @args) = @_;
 
-	my $args = join ' ', @args;
+    my $args = join ' ', @args;
 
-	return
-		SCALAR   { `$exe log $args` }
-		ARRAYREF { `$exe log $args` }
-		HASHREF  {
-			my $logs = `$exe log $args`;
-			my @logs = split /^-+\n/xms, $logs;
-			shift @logs;
-			my $num = @logs;
-			my %log;
-			for my $log (@logs) {
-				my ($details, $description) = split /\n\n?/, $log, 2;
-				$details =~ s/^\s*(.*?)\s*/$1/;
-				my @details = split /\s+\|\s+/, $details;
-				$details[0] =~ s/^r//;
-				$log{$num--} = {
-					rev    => $details[0],
-					Author => $details[1],
-					Date   => $details[2],
-					description => $description,
-				},
-			}
-			return \%log;
-		}
+    return
+        SCALAR   { scalar `$exe log $args` }
+        ARRAYREF {
+            my @raw_log = `$exe log $args`;
+            my @log;
+            my $line = '';
+            for my $raw (@raw_log) {
+                if ( $raw eq ( '-' x 72 ) . "\n"  && $line ) {
+                    CORE::push @log, $line;
+                    $line = '';
+                }
+                elsif ( $raw ne ( '-' x 72 ) . "\n"  ) {
+                    $line .= $raw;
+                }
+
+            }
+            return \@log;
+        }
+        HASHREF  {
+            my $logs = `$exe log $args`;
+            my @logs = split /^-+\n/xms, $logs;
+            shift @logs;
+            my $num = @logs;
+            my %log;
+            for my $log (@logs) {
+                my ($details, $description) = split /\n\n?/, $log, 2;
+                $description =~ s/\s+\Z//xms;
+                $details =~ s/^\s*(.*?)\s*/$1/;
+                my @details = split /\s+\|\s+/, $details;
+                $details[0] =~ s/^r//;
+                $log{$num--} = {
+                    rev    => $details[0],
+                    Author => $details[1],
+                    Date   => $details[2],
+                    description => $description,
+                },
+            }
+            return \%log;
+        }
 }
 
 sub versions {
-	my ($self, $file, $before_version, $max) = @_;
+    my ($self, $file, $before_version, $max) = @_;
 
-	$file = file($file);
-	local $CWD = -d $file ? $file : $file->parent;
-	my %logs = %{ $self->log(-d $file ? '.' : $file->basename, $max ? "--limit $max" : '') };
-	my @versions;
+    $file = file($file);
+    local $CWD = -d $file ? $file : $file->parent;
+    my %logs = %{ $self->log(-d $file ? '.' : $file->basename, $max ? "--limit $max" : '') };
+    my @versions;
 
-	for my $log (sort {$a <=> $b} keys %logs) {
-		push @versions, $logs{$log}{rev};# if $before_version && $logs{$log}{rev} <= $before_version;
-	}
+    for my $log (sort {$a <=> $b} keys %logs) {
+        push @versions, $logs{$log}{rev};# if $before_version && $logs{$log}{rev} <= $before_version;
+    }
 
-	return @versions;
+    return @versions;
 }
 
 1;
@@ -143,7 +159,7 @@ VCS::Which::Plugin::Subversion - The Subversion plugin for VCS::Which
 
 =head1 VERSION
 
-This documentation refers to VCS::Which::Plugin::Subversion version 0.2.0.
+This documentation refers to VCS::Which::Plugin::Subversion version 0.3.0.
 
 =head1 SYNOPSIS
 
